"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CLIOptions = void 0;
const tslib_1 = require("tslib");
const FS = tslib_1.__importStar(require("fs"));
const OS = tslib_1.__importStar(require("os"));
const Path = tslib_1.__importStar(require("path"));
const clime_1 = require("clime");
const prompts_1 = tslib_1.__importDefault(require("prompts"));
const tiva_1 = require("tiva");
const yaml_1 = tslib_1.__importDefault(require("yaml"));
const config_1 = require("../config");
const main_1 = require("../main");
const shared_1 = require("../shared");
const WORKSPACE_PATH_DEFAULT = Path.resolve(OS.homedir(), '.config', 'makescript', 'agent');
const GENERATE_CONFIG_DEFAULT = false;
const CONFIG_FILE_NAME = 'agent.yaml';
class CLIOptions extends clime_1.Options {
}
tslib_1.__decorate([
    clime_1.option({
        flag: 'g',
        description: 'Whether only to generate makescript agent config file.',
        toggle: true,
        default: GENERATE_CONFIG_DEFAULT,
    }),
    tslib_1.__metadata("design:type", Boolean)
], CLIOptions.prototype, "generateConfigOnly", void 0);
tslib_1.__decorate([
    clime_1.option({
        flag: 'w',
        description: 'The path for makescript agent to work.',
        default: WORKSPACE_PATH_DEFAULT,
    }),
    tslib_1.__metadata("design:type", clime_1.Castable.Directory)
], CLIOptions.prototype, "workspace", void 0);
exports.CLIOptions = CLIOptions;
let default_1 = class default_1 extends clime_1.Command {
    async execute({ generateConfigOnly, workspace }) {
        let configFilePath = Path.join(workspace.fullName, CONFIG_FILE_NAME);
        let configFileExists = FS.existsSync(configFilePath);
        if (configFileExists && generateConfigOnly) {
            console.warn(`The config file "${configFilePath}" has existed, so the new config has not generated.`);
            return;
        }
        if (!configFileExists) {
            let dirname = workspace.fullName;
            if (!FS.existsSync(dirname)) {
                FS.mkdirSync(dirname);
            }
            let answer = await prompts_1.default([
                {
                    type: 'text',
                    name: 'hostURL',
                    message: 'Please enter the makescript host secret url to join',
                    validate: value => /^https?:\/\/.+$/.test(value),
                },
                {
                    type: 'text',
                    name: 'namespace',
                    message: 'What namespace do you want to register as',
                },
                {
                    type: 'text',
                    name: 'repoURL',
                    message: 'Please enter the git repo url of the scripts',
                    validate: value => /^(https?:\/\/.+)|(\w+\.git)$/.test(value),
                },
            ]);
            // There is a bug (or unhandled behavior) with 'prompts'.
            // When user press CTRL + C , program will continue to execute with empty answers.
            // https://github.com/terkelg/prompts/issues/252
            if (!answer.hostURL || !answer.namespace || !answer.repoURL) {
                return;
            }
            let yamlConfigContent = config_1.generateYamlConfig({
                'makescript-secret-url': answer.hostURL,
                'scripts-repo-url': answer.repoURL,
                namespace: answer.namespace,
                proxy: undefined,
            });
            FS.writeFileSync(configFilePath, yamlConfigContent);
        }
        if (generateConfigOnly) {
            return;
        }
        let yamlConfigContent = FS.readFileSync(configFilePath).toString();
        let configFileContent = yaml_1.default.parse(yamlConfigContent);
        let tiva = new tiva_1.Tiva();
        shared_1.logger.info('Checking config file ...');
        try {
            await tiva.validate({ module: '@makeflow/makescript-agent', type: 'ConfigFile' }, configFileContent);
            let config = config_1.transformConfig(configFileContent, workspace.fullName);
            await main_1.main(tiva, config);
        }
        catch (error) {
            if (error.diagnostics) {
                shared_1.logger.error(`Config file structure does not match:\n${error.diagnostics}`);
            }
            else {
                shared_1.logger.error(`Unknown error occurred:\n${error.message}`);
            }
            process.exit(1);
        }
    }
};
tslib_1.__decorate([
    clime_1.metadata,
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [CLIOptions]),
    tslib_1.__metadata("design:returntype", Promise)
], default_1.prototype, "execute", null);
default_1 = tslib_1.__decorate([
    clime_1.command()
], default_1);
exports.default = default_1;
//# sourceMappingURL=default.js.map